﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using MbUnit.Framework;

namespace TestingLinqToSQLMockA.Tests
{
    [TestFixture]
    public class ProductRepositoryTests
    {
        //Accessing these via the DataConText to save implementing the methods on the ProductRepository.
        //We could have used ADO.net
        private void InsertCategory()
        {
            Category c = new Category();
            c.Title = "Microsoft Software";
            c.Description = "All the latest Microsoft releases.";

            HowToUnitTestDataContext db = new HowToUnitTestDataContext();
            db.Categories.Add(c);
            db.SubmitChanges();
        }

        private void InsertProduct(Category c)
        {
            HowToUnitTestDataContext db = new HowToUnitTestDataContext();

            Product p = new Product();
            p.Title = "Visual Studio 2008";
            p.Description = "IDE";
            p.Price = 1000;
            db.Products.Add(p);
            db.SubmitChanges();

            ProductCategory pc = new ProductCategory();
            pc.CategoryID = c.id;
            pc.ProductID = p.id;

            db.ProductCategories.Add(pc);
            db.SubmitChanges();
        }

        LinqProductRepository m_ProductRepository = new LinqProductRepository();

        [Test]
        public void GetCategories_NoContent_ReturnEmptyList()
        {
            List<Category> categories = m_ProductRepository.GetCategories();
            Assert.IsNotNull(categories);
            Assert.AreEqual(0, categories.Count);
        }

        [Test]
        [RollBack]
        public void GetCategories_SimpleContent_ReturnPopulatedList()
        {
            InsertCategory();

            List<Category> categories = m_ProductRepository.GetCategories();
            Assert.IsNotNull(categories);
            Assert.AreEqual(1, categories.Count);
            Assert.AreEqual("Microsoft Software", categories[0].Title);
        }

        [Test]
        [RollBack]
        public void GetCategoryByName_NameOfValidCategory_ReturnCategoryObject()
        {
            InsertCategory();

            Category c = m_ProductRepository.GetCategory("Microsoft Software");

            Assert.AreEqual("Microsoft Software", c.Title);
            Assert.AreEqual("All the latest Microsoft releases.", c.Description);
        }

        [Test]
        [RollBack]
        public void GetProductsForCategory_ValidCategory_EmptyList()
        {
            InsertCategory();

            Category c = m_ProductRepository.GetCategory("Microsoft Software");

            List<Product> products = m_ProductRepository.GetProductsForCategory(c);
            Assert.AreEqual(0, products.Count);
        }

        [Test]
        [RollBack]
        public void GetProductsForCateogry_ValidCategoryWithProduct_PopulatedList()
        {
            InsertCategory();

            Category c = m_ProductRepository.GetCategory("Microsoft Software");
            InsertProduct(c);

            List<Product> products = m_ProductRepository.GetProductsForCategory(c);
            Assert.AreEqual(1, products.Count);
            Assert.AreEqual("Visual Studio 2008", products[0].Title);
        }

        [Test]
        [RollBack]
        public void GetProductByID_ValidProductID_ReturnProductID()
        {
            InsertCategory();

            Category c = m_ProductRepository.GetCategory("Microsoft Software");
            InsertProduct(c);

            List<Product> products = m_ProductRepository.GetProductsForCategory(c);
            Assert.AreEqual(1, products.Count);

            Product p = m_ProductRepository.GetProduct(products[0].id);
            Assert.AreEqual(p.Title, products[0].Title);
        }
    }
}
